@@ -299,6 +299,15 @@ BOOL WINAPI RoOriginateError(HRESULT error, HSTRING message)
     return FALSE;
 }
 
+/***********************************************************************
+ *      RoSetErrorReportingFlags (combase.@)
+ */
+HRESULT WINAPI RoSetErrorReportingFlags(UINT32 flags)
+{
+    FIXME("(%08x): stub\n", flags);
+    return S_OK;
+}
+
 /***********************************************************************
  *      CleanupTlsOleState (combase.@)
  */
